﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Collections;
using UdpSrcCom;
using DirectShowLib;
using System.Runtime.InteropServices;

namespace Player
{
    // Simple player
    public partial class PlayerWindow : Form
    {

        private const int GRAPH_NOTIFY = 0x00008002;
        public delegate void delegateNoPrm();
       
        IPAddress NicIp;
        int NicIndex;
        ushort Port;
        IPAddress MulticastAddress;

        private IBaseFilter m_UdpSource = null;                 // UDP source and demultiplexer filter 
        private IUdpMulticast m_IUdpMulticast = null;           // Interface exposed by UDP source and demultiplexer filter, used to configure filter.
        private IFilterGraph2 m_FilterGraph = null;
        private IMediaControl m_mediaCtrl = null;
        private IMediaEventEx m_mediaEvent = null;
        
        private TSParser m_parser = null;
        private IVideoWindow m_videoWindow = null;
        private IBaseFilter m_vmr9 = null;                      // VMR9 renderer

        public PlayerWindow(IPAddress nicIp, int nicIndex, ushort port, IPAddress multicastIp )
        {
            InitializeComponent();
            NicIp = nicIp;
            NicIndex = nicIndex;
            Port = port;
            MulticastAddress = multicastIp;
            this.Text = multicastIp.ToString()+":"+port.ToString();
            setupGraph();
        }

        protected override void WndProc(ref Message m)
        {
            if (m.Msg == GRAPH_NOTIFY)
            {
              
                this.Invoke(new delegateNoPrm(eventHandler));
            }
            else
            {
                base.WndProc(ref m);
            }
        }

        private void eventHandler()
        {
            IntPtr p1= IntPtr.Zero, p2 = IntPtr.Zero;
            int hr = 0;
            EventCode code=0;
            do
            {
                 hr = -1;
                lock (this)
                {
                    if (m_mediaEvent != null)
                        hr = m_mediaEvent.GetEvent(out code, out p1, out p2, 0);
                }      
                
                if (hr < 0)
                    break;
               
                switch ((int)code)
                {
                    case TSParser.EC_PARSER_COMPLETE:
                            startPlaying();
                        break;

                    default:
                        break;
                }

                lock (this)
                {
                    if (m_mediaEvent!=null)
                        m_mediaEvent.FreeEventParams(code, p1, p2);
                }
              
                   
            } while (hr == 0);
        }

        private void setupGraph()
        {
            int hr;
            try
            {
                m_FilterGraph = (IFilterGraph2)new FilterGraph();

                m_videoWindow = (IVideoWindow)m_FilterGraph; 

                m_UdpSource = (IBaseFilter)new UdpSourceFilter();   // Udp source and TS demultiplexer filter.

                m_IUdpMulticast = (IUdpMulticast)m_UdpSource;       // Query UdpMulticast source filter for IUdpMulticast interface.

                m_parser = new TSParser(m_IUdpMulticast);           // Parsing helper class.

                hr = m_IUdpMulticast.SetMulticastIp(BitConverter.ToUInt32(MulticastAddress.GetAddressBytes(), 0)); // Configure Udp Source Filter
                DsError.ThrowExceptionForHR(hr);

                hr = m_IUdpMulticast.SetPort(Port);
                DsError.ThrowExceptionForHR(hr);

                hr = m_IUdpMulticast.SendPTSToH264Dec(true);
                DsError.ThrowExceptionForHR(hr);

                hr = m_IUdpMulticast.SetNicIp(BitConverter.ToUInt32(NicIp.GetAddressBytes(), 0));
                DsError.ThrowExceptionForHR(hr);

                if (NicIndex >= 0)
                {
                    hr = m_IUdpMulticast.SetNicIndex((uint)NicIndex);
                    DsError.ThrowExceptionForHR(hr);
                }

                hr = m_FilterGraph.AddFilter(m_UdpSource, "Udp Source and TS Parser/Demultiplexer Filter (MulticastTV) "+MulticastAddress.ToString());// Add source filter
                DsError.ThrowExceptionForHR(hr);

                m_mediaEvent = (IMediaEventEx)m_FilterGraph;

                m_mediaEvent.SetNotifyWindow(this.Handle, GRAPH_NOTIFY, IntPtr.Zero);

                m_mediaCtrl = (IMediaControl)m_FilterGraph;

                if(hr==0)
                    m_UdpSource.Run(0);

            }
            catch
            {
                this.Invoke(new delegateNoPrm(cleanGraph));
            }
        }

        private void startPlaying()
        {
            int hr;

            m_parser.GetInfo();
            m_UdpSource.Stop();

            if (m_parser.ProgrameList.Count == 0)
            {
                stopPlaying();
                return;
            }

            ICaptureGraphBuilder2 gbuild2 = null;
            gbuild2 = (ICaptureGraphBuilder2)new CaptureGraphBuilder2();
            hr = gbuild2.SetFiltergraph(m_FilterGraph);
            DsError.ThrowExceptionForHR(hr);

            int program = 0; // Select first program

            // Try to find a program that contains video.
            for (int i = 0; i < m_parser.ProgrameList.Count; i++)
            {
                if (m_parser.ProgrameList[i].Video.Count > 0)
                {
                    program = i;
                    break;
                }
            }

            try
            {
                byte renderresult = 0;
                bool renderAudio = false;
                if (m_parser.ProgrameList[program].Audio.Count > 0)
                    renderAudio = true;

                // Tell demultiplexer which packets contains video and audio.
                if (m_parser.ProgrameList[program].Video.Count > 0)          // Render both pins.
                {
                    if (renderAudio)
                        m_IUdpMulticast.ConfigureDemultiplexer(m_parser.ProgrameList[program].Video[0].PID, m_parser.ProgrameList[program].Audio[0].PID, 0);
                    else
                        m_IUdpMulticast.ConfigureDemultiplexer(m_parser.ProgrameList[program].Video[0].PID, 0, 0);

                    m_vmr9 = (IBaseFilter)new VideoMixingRenderer9();       // Add and configure VMR-9 in Windowed mode.
                    hr = m_FilterGraph.AddFilter(m_vmr9, "Video Mixing Renderer 9");
                    //IVMRFilterConfig9 fc = (IVMRFilterConfig9)m_vmr9;
                    //fc.SetRenderingMode(VMR9Mode.Windowed);
  
                    hr = gbuild2.RenderStream(null, MediaType.Video, m_UdpSource, null, m_vmr9);
                    if (hr == 0)
                        renderresult |= TSParser.VIDEO_PIN_SUCCESS;

                    DsError.ThrowExceptionForHR(hr);

                    if (renderAudio)
                    {
                        hr = gbuild2.RenderStream(null, MediaType.Audio, m_UdpSource, null, null);
                        if (hr == 0)
                            renderresult |= TSParser.AUDIO_PIN_SUCCESS;
                        DsError.ThrowExceptionForHR(hr);
                    }

                    ConfigureVideoWindow(this, m_parser.ProgrameList[program].Video[0].Width,
                                        m_parser.ProgrameList[program].Video[0].Height,
                                        m_parser.ProgrameList[program].Video[0].AspectRatioX,
                                        m_parser.ProgrameList[program].Video[0].AspectRatioY,
                                        m_parser.ProgrameList[program].Video[0].Type);

                }
                else // Render only audio pin.
                {
                    if (renderAudio)
                    {
                        m_IUdpMulticast.ConfigureDemultiplexer(0, m_parser.ProgrameList[program].Audio[0].PID, 0);

                        hr = gbuild2.RenderStream(null, MediaType.Audio, m_UdpSource, null, null);
                        if (hr == 0)
                            renderresult |= TSParser.AUDIO_PIN_SUCCESS;

                        DsError.ThrowExceptionForHR(hr);
                    }
                }


                m_IUdpMulticast.ReportRenderingSuccess(renderresult);   // Report rendering results to filter
                hr = m_mediaCtrl.Run();                                 // Now run the graph.
                DsError.ThrowExceptionForHR(hr);
            }
            catch
            {
                cleanGraph();
            }
            if (gbuild2 != null)
                Marshal.ReleaseComObject(gbuild2);
        }

        private void stopPlaying()
        {
            if (m_mediaCtrl != null)
                m_mediaCtrl.Stop();
            cleanGraph();
        }

        private void ConfigureVideoWindow(Control hWin, int pictWidth, int pictHeight, uint pictArx, uint pictAry, ushort decType)
        {
            int hr;

            hr = m_videoWindow.put_Owner(hWin.Handle);                        // Set the output window
            DsError.ThrowExceptionForHR(hr);

            hr = m_videoWindow.put_WindowStyle((WindowStyle.Child | WindowStyle.ClipChildren // Set the window style
                                            | WindowStyle.ClipSiblings));
            DsError.ThrowExceptionForHR(hr);

            hr = m_videoWindow.put_Visible(OABool.True);                      // Make the window visible
            DsError.ThrowExceptionForHR(hr);

            hr = m_videoWindow.put_MessageDrain(hWin.Handle);   // Relay messages to main form
            DsError.ThrowExceptionForHR(hr);

            setVideoDimensions(pictWidth,pictHeight,pictArx,pictAry,decType);            // Position the playing location
        }

        private void setVideoDimensions(int pictWidth, int pictHeight, uint pictArx, uint pictAry, ushort decType)
        {
            if (pictAry != 0)
                this.ClientSize = new Size((pictWidth * 1000 * (int)pictArx / (int)pictAry) / 1000, pictHeight);

            int winHeight = this.ClientRectangle.Height;
            int winWidth = this.ClientRectangle.Width;
            int arv = (pictWidth * 1000) / pictHeight;
            int arp = (winWidth * 1000) / winHeight;

            switch (decType)
            {
                case (ushort)StreamType.VideoMpeg1:
                case (ushort)StreamType.VideoMpeg2:
                    if (pictAry != 0)
                        arv = (int)pictArx * 1000 / (int)pictAry;
                    break;

                case (ushort)StreamType.VIdeoH264: // h.264 has PAR information !
                    if (pictAry != 0 && pictHeight != 0)
                        arv = pictWidth * (int)pictArx* 1000 / ((int)pictAry * pictHeight);
                    break;
            }

            int pict_width;
            int pict_height;
            int x_ofs;
            int y_ofs;

            if (arp > arv)
            {
                pict_width = arv * winHeight / 1000;
                pict_height = winHeight;
                x_ofs = (winWidth - pict_width) / 2;
                y_ofs = 0;
            }
            else
            {
                pict_width = winWidth;
                pict_height = winWidth * 1000 / arv;
                x_ofs = 0;
                y_ofs = (winHeight - pict_height) / 2;
            }

            m_videoWindow.SetWindowPosition(
            this.ClientRectangle.Left + x_ofs,
            this.ClientRectangle.Top + y_ofs,
            pict_width,
            pict_height
           );
        }

        private void cleanGraph()
        {  
            lock (this)
            {

                if (m_mediaCtrl != null)
                {
                    Marshal.ReleaseComObject(m_mediaCtrl);
                    m_mediaCtrl = null;
                }
            
                if (m_IUdpMulticast != null)
                {
                    Marshal.ReleaseComObject(m_IUdpMulticast);
                    m_IUdpMulticast = null;
                }

                if (m_UdpSource != null)
                {
                    Marshal.ReleaseComObject(m_UdpSource);
                    m_UdpSource = null;
                }

                if (m_videoWindow != null)
                {
                    Marshal.ReleaseComObject(m_videoWindow);
                    m_videoWindow = null;
                }

                if (m_vmr9 != null)
                {
                    Marshal.ReleaseComObject(m_vmr9);
                    m_videoWindow = null;
                    m_vmr9 = null;
                }

                if (m_FilterGraph != null)
                {
                    Marshal.ReleaseComObject(m_FilterGraph);
                    m_FilterGraph = null;
                }

                if (m_mediaEvent != null)
                {
                    Marshal.ReleaseComObject(m_mediaEvent);
                    m_mediaEvent = null;
                }


            }
        }

        private void PlayerWindow_FormClosing(object sender, FormClosingEventArgs e)
        {
            stopPlaying();
        }

     }
}
