﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;

namespace MulticastTV
{
    class NetworkInterfaces
    {
        NetworkInterface[] nics = NetworkInterface.GetAllNetworkInterfaces();

        public List<string> getNames()
        {
            List<string> ips = new List<string>();

            foreach (NetworkInterface adapter in nics)
            {
                if (!ips.Contains(adapter.Name))
                    ips.Add(adapter.Name);
            }
            return ips;
        }

        public string getIpFromName(string name)
        {
            foreach (NetworkInterface adapter in nics)
            {
                if (adapter.Name.Equals(name))
                {
                    IPInterfaceProperties properties = adapter.GetIPProperties();
                    UnicastIPAddressInformationCollection ipv4 = properties.UnicastAddresses;
                    foreach (UnicastIPAddressInformation uip in ipv4)
                        if (uip.Address.AddressFamily.ToString() == ProtocolFamily.InterNetwork.ToString())
                            return uip.Address.ToString();
                    // This nic hasn't assigned an ipv4 address
                }
            }
            return "";

        }

        public int getIndexFromName(string name)
        {
            if (name.Length == 0)
            { return 0; }

            foreach (NetworkInterface adapter in nics)
            {
                if (adapter.Name.Equals(name))
                {
                    IPInterfaceProperties properties = adapter.GetIPProperties();
                    try
                    {
                        int index = properties.GetIPv4Properties().Index;
                        return index;
                    }
                    // 0 is normally for localhost
                    catch { return -1; }
                }
            }
            return 0;
        }

        public NetworkInterface getAdapter(int index)
        {
            foreach (NetworkInterface adapter in nics)
            {
                IPInterfaceProperties properties = adapter.GetIPProperties();
                IPv4InterfaceProperties iv4 = properties.GetIPv4Properties();
                if (iv4 != null && iv4.Index == index)
                    return adapter;
            }
            return null;
        }

        public bool supportsMulticast(int index)
        {
            foreach (NetworkInterface adapter in nics)
            {
                IPInterfaceProperties properties = adapter.GetIPProperties();
                IPv4InterfaceProperties iv4 = properties.GetIPv4Properties();
                if (iv4 != null && iv4.Index == index)
                    return adapter.SupportsMulticast;
            }
            return false;
        }
    }
}
