﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Net;
using System.Threading;
using System.Runtime.InteropServices;
using MulticastTV;
using UdpSrcCom;
using DirectShowLib;


namespace MctvPlaylistEditor.Scanner
{
    public enum GraphState
    {
        Stopped,
        Paused,
        Running,
        Exiting,
        NoGraph,
        Preparing
    }

    class Scanner //: IDisposable

    {
        private IBaseFilter m_UdpSource = null;                 // UDP source and demultiplexer filter 
        private IUdpMulticast m_IUdpMulticast = null;           // Interface exposed by UDP source and demultiplexer filter, used to configure filter.
        private IFilterGraph2 m_FilterGraph = null;

        IPAddress m_NicIp = IPAddress.Any;
        int m_NicIndex = 0;

        private NetworkInterfaces m_nicInfo = new NetworkInterfaces();
        public TSParser m_parser = null;
        private IntPtr m_hEvent;

        public ushort m_multicastPort = 0;
        public IPAddress m_multicastIp;
        public IMediaEvent m_mediaEvent = null;
        public ManualResetEvent m_mre = null;
        public ManualResetEvent m_mreEventLoop = null;
        public volatile GraphState m_State = GraphState.NoGraph;

        public int m_Timeout = 0;


        public Scanner(IPAddress NicIp, int NicIndex, int timeout)
        {
            m_NicIp = NicIp;
            m_NicIndex = NicIndex;
            m_Timeout = timeout;
        }

        ~Scanner()
        {
            //Dispose(false);
        }

        //public void Dispose()
        //{
        //    Dispose(true);
        //    GC.SuppressFinalize(this);
        //}

        // protected void Dispose(bool Disposing)
        // {
        //    if(!IsDisposed)
        //    {
        //        if(Disposing)
        //        {
        //         //Clean Up managed resources
        //        }
            
        //      //Clean up unmanaged resources
        //        DestroyGraph();
        //     }
        //     IsDisposed=true;
        // }

        public void initGraph()
        { 
            int hr = 0;
            if (m_FilterGraph == null)
                  m_FilterGraph = (IFilterGraph2)new FilterGraph();

            if (m_mediaEvent == null)
            {
                m_mediaEvent = (IMediaEvent)m_FilterGraph;
                hr = m_mediaEvent.GetEventHandle(out m_hEvent);
                DsError.ThrowExceptionForHR(hr);
                m_mre = new ManualResetEvent(false);
                m_mre.SafeWaitHandle = new Microsoft.Win32.SafeHandles.SafeWaitHandle(m_hEvent, true);
            }

            if (m_mreEventLoop==null)
                m_mreEventLoop=new ManualResetEvent(false);
        }

        public void startGraph(IPAddress ip, ushort port)
        {
            int hr = 0;
            m_State = GraphState.Preparing;
            m_multicastIp = ip;
            m_multicastPort = port;

            try
            {
                lock (this)
                {
                    if (m_UdpSource == null)
                    m_UdpSource = (IBaseFilter)new UdpSourceFilter();   // Udp source and TS demultiplexer filter.
  
                   // Marshal.AddRef(Marshal.GetComInterfaceForObject(m_UdpSource, typeof(IBaseFilter)));
     
                    if (m_IUdpMulticast == null)
                    m_IUdpMulticast = (IUdpMulticast)m_UdpSource;       // Query UdpMulticast source filter for IUdpMulticast interface.
 
                   // Marshal.AddRef(Marshal.GetComInterfaceForObject(m_IUdpMulticast, typeof(IUdpMulticast)));

                    m_parser = new TSParser(m_IUdpMulticast);           // Parsing helper class.

                    hr = m_IUdpMulticast.SetMulticastIp(BitConverter.ToUInt32(ip.GetAddressBytes(), 0)); // Configure Udp Source Filter
                    DsError.ThrowExceptionForHR(hr);

                    hr = m_IUdpMulticast.SetPort(port);
                    DsError.ThrowExceptionForHR(hr);

                    if (m_NicIp != IPAddress.Any)
                        hr = m_IUdpMulticast.SetNicIp(BitConverter.ToUInt32(m_NicIp.GetAddressBytes(), 0));
                    else
                        hr = m_IUdpMulticast.SetNicIp(0);
                    DsError.ThrowExceptionForHR(hr);

                    hr = m_IUdpMulticast.SetMode(TSParser.PARSING_MODE);
                    DsError.ThrowExceptionForHR(hr);

                    if (m_NicIndex >= 0)
                    {
                        hr = m_IUdpMulticast.SetNicIndex((uint)m_NicIndex);
                        DsError.ThrowExceptionForHR(hr);
                    }

                    hr = m_FilterGraph.AddFilter(m_UdpSource, "Udp Source and TS Parser Filter (MulticastTV)");// Add source filter
                    DsError.ThrowExceptionForHR(hr);

                    hr = m_UdpSource.Run(0);                          // Run Udp Source Filter - NOT THE GRAPH !
                    if (hr == (int)TSParser.ERR_RECV_IPAGN_AND_IPV4_BIND_FAILED)
                    {
                        return;
                    }
                    DsError.ThrowExceptionForHR(hr);
                    m_State = GraphState.Running;
                }
            }
            catch
            {
                cleanGraph();
            }
        }

        public void stopFilter()
        {
            lock (this)
            {
                if (m_UdpSource != null && m_State == GraphState.Running)
                {
                    m_UdpSource.Stop();
                    m_State = GraphState.Stopped;
                }
            }
        }

        public void cleanGraph()
        {
            lock (this)
            {

                if (m_parser != null)
                {
                    m_parser.ProgrameList.Clear();
                    m_parser = null;
                }

                if (m_UdpSource != null)
                    m_FilterGraph.RemoveFilter(m_UdpSource);

                if (m_IUdpMulticast != null)
                {
                    Marshal.ReleaseComObject(m_IUdpMulticast);
                    m_IUdpMulticast = null;
                }

                if (m_UdpSource != null)
                {
                    Marshal.ReleaseComObject(m_UdpSource);
                    m_UdpSource = null;
                }
            }
        }

         public void DestroyGraph()
        {
            lock (this)
            {
                m_State = GraphState.Exiting; // Exit event loop
                if (m_mre != null && !m_mre.SafeWaitHandle.IsInvalid)
                {
                    try
                    {
                        m_mre.Set();
                    }
                    catch { }
                }
            }

            if (m_mediaEvent != null)
            {
                m_mreEventLoop.WaitOne(); // Wait for event loop to exit
                Marshal.ReleaseComObject(m_mediaEvent);
                m_mediaEvent = null;
                m_hEvent = IntPtr.Zero;
                try
                {
                    m_mre.Close();
                    m_mre = null;
                }
                catch { m_mre = null; }
            }


            if (m_FilterGraph != null)
            {
                Marshal.ReleaseComObject(m_FilterGraph);
                m_FilterGraph = null;
                m_mediaEvent = null;
            }

            lock (this)
            {
                m_State = GraphState.NoGraph;
            }
        }

         private static void removeAllFilters(IGraphBuilder graphBuilder)
         {
             int hr = 0;
             IEnumFilters enumFilters;
             ArrayList filtersArray = new ArrayList();

             if (graphBuilder == null)
                 return;

             hr = graphBuilder.EnumFilters(out enumFilters);
             DsError.ThrowExceptionForHR(hr);

             try
             {
                 IBaseFilter[] filters = new IBaseFilter[1];
                 IntPtr fetched = IntPtr.Zero;

                 while (enumFilters.Next(filters.Length, filters, fetched) == 0)
                 {
                     filtersArray.Add(filters[0]);
                 }
             }
             finally
             {
                 Marshal.ReleaseComObject(enumFilters);
             }

             foreach (IBaseFilter filter in filtersArray)
             {
                 hr = graphBuilder.RemoveFilter(filter);
                 Marshal.ReleaseComObject(filter);
             }
         }
    }
}
