﻿//Copyright (c) 2010-2011, Uroš Steklasa
//All rights reserved.
//
//Redistribution of this file (UdpSourceTsParser.cs) and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//	* Redistributions may not be sold, nor may they be used in a commercial product or activity.
//	* Redistributions of source code and/or in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//THIS SOFTWARE IS PROVIDED BY Uroš Steklasa "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

using System;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;

// Interop and helper clases.
namespace UdpSrcCom
{

    #region Interop
    #region Interface
    // IUdpMulticast as a COM interface.
    [Guid("C7953E93-2C62-428D-A2F0-4F73F07994DC"),
    InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    public interface IUdpMulticast
    {
        [PreserveSig]
        int GetMulticastIp(
        [Out] out UInt32 plMulticastip
        );

        [PreserveSig]
        int SetMulticastIp(
        [In] UInt32 lMulticastip
        );

        [PreserveSig]
        int GetNicIp(
            [Out] out UInt32 plNICip
            );

        [PreserveSig]
        int SetNicIp(
            [In] UInt32 lNICip
            );

        [PreserveSig]
        int SetNicIndex(
		[In] UInt32 lNicIndex 
		);

        [PreserveSig]
        int GetPort(
            [Out] out ushort psPort
            );

        [PreserveSig]
        int SetPort(
            [In] ushort sPort
            );

        [PreserveSig]
        int GetAudioPid(
            [Out] out ushort piAudpid
            );

        [PreserveSig]
        int SetAudioPid(
            [In] ushort iAudpid
            );

        [PreserveSig]
        int GetVideoPid(
            [Out] out ushort piVidpid
            );

        [PreserveSig]
        int SetVideoPid(
            [In]  ushort iVidpid
            );

        [PreserveSig]
        	int GetSubtitlesPid(
		    [Out] out short piSpid
		);

        [PreserveSig]
	        int SetSubtitlesPid(
		    [In] ushort iSpid
		);

        [PreserveSig]
        int SetRecFile(
            [In, MarshalAs(UnmanagedType.LPStr)] string pszFilePath
            );

        [PreserveSig]
        int SetSrcFile(
            [In, MarshalAs(UnmanagedType.LPStr)] string pszFilePath
            );

        /// <summary>
        /// Set which video, audio and subtitle streams should be demultiplexed.
        /// </summary>
        /// <param name="iVideoPid"></param>
        /// <param name="iAudioPid"></param>
        /// <param name="iSubtitlePid"></param>
        /// <returns></returns>
        [PreserveSig]
        int ConfigureDemultiplexer(
            [In] UInt16 iVideoPid,
            [In] UInt16 iAudioPid,
            [In] UInt16 iSubtitlePid
            );
  
        //TODO: ConfigureParserManualVideo, ConfigureParserManualAudio     

        /// <summary>
        /// Tell filter which pins are successfully rendered.
        /// </summary>
        /// <param name="iPinCode"></param>
        /// <returns></returns>
        [PreserveSig]
        int ReportRenderingSuccess(
            [In] System.Byte iPinCode
            );

        /// <summary>
        /// Get all necessary data that is needed for creating directshow graph.
        /// This function must be called after received EC_PARSER_COMPLETE message !
        /// </summary>
        /// <param name="iProgIndex"></param>
        /// <param name="iStreamIndex"></param>
        /// <param name="pStreamInfo"></param>
        /// <returns></returns>
        [PreserveSig]
        int GetParsedData(
            [In] Int32 iProgIndex,
            [In] Int32 iStreamIndex,
            [Out, In] ref parser_StreamInfo pStreamInfo
            );

        /// <summary>
        /// Get number of programs - PAT table.
        /// </summary>
        /// <param name="piCount"></param>
        /// <returns></returns>
        [PreserveSig]
        int GetProgCount(
             [Out] out Int32 piCount
             );

        /// <summary>
        /// Get number of streams for particular programme. 
        /// </summary>
        /// <param name="iProgIndex"></param>
        /// <param name="piCount"></param>
        /// <returns></returns>
        [PreserveSig]
        int GetPidCount(
            [In] Int32 iProgIndex,
            [Out] out Int32 piCount
         );

        [PreserveSig]
        int GetSubDesc(
            [In] Int32 iProgIndex,
            [In] Int32 iStreamIndex,
            [In] Int32 iSubIndex,
            [Out, In] ref parser_SubtitleDesc pSubtitleDesc
        );

        [PreserveSig]
        int GetTtxDesc(
            [In] Int32 iProgIndex,
            [In] Int32 iStreamIndex,
            [In] Int32 iTtxIndex,
            [Out, In] ref parser_TeletextDesc pTeletextDesc
            );

        [PreserveSig]
        int GetVideoInfo(
            [In] Int32 iProgIndex,
            [In] Int32 iStreamIndex,
            [Out, In] ref parser_VideoInfo pVidInfo
            );

        [PreserveSig]
        int GetAudioInfo(
            [In] Int32 iProgIndex,
            [In] Int32 iStreamIndex,
            [Out, In] ref parser_AudioInfo pAudInfo
            );

        /// <summary>
        /// Get PID of each PAT table.
        /// </summary>
        /// <param name="iProgIndex"></param>
        /// <param name="pProgNumber"></param>
        /// <param name="pPatPid"></param>
        /// <returns></returns>
        [PreserveSig]
        int GetPATPid(
            [In] Int32 iProgIndex,
            [Out] out UInt16 pProgNumber,
            [Out] out UInt16 pPatPid
            );

        /// <summary>
        /// Get stream type and PID from PAT table.
        /// </summary>
        /// <param name="iProgIndex"></param>
        /// <param name="iStreamIndex"></param>
        /// <param name="pPid"></param>
        /// <param name="pType"></param>
        /// <returns></returns>
        [PreserveSig]
        int GetPMTType(
            [In] Int32 iProgIndex,
            [In] Int32 iStreamIndex,
            [Out] out UInt16 pPid,
            [Out] out Int32 pType
            );

        /// <summary>
        /// Set if filter will either be used for playback or just parsing.
        /// </summary>
        /// <param name="mode"></param>
        /// <returns></returns>
        [PreserveSig]
        int SetMode(
            [In] byte mode
            );

        /// <summary>
        /// Get number of service descriptors.
        /// </summary>
        /// <param name="pCount"></param>
        /// <returns></returns>
        [PreserveSig]
        int ServicesCount(
            [Out] out Int32 pCount
            );

        /// <summary>
        /// Get service descriptor.
        /// This function must be called after received EC_MSG_SERVICE_LIST_RECEIVED message !
        /// </summary>
        /// <param name="iIndex">Index</param>
        /// <param name="pProvider">Provider (String builder buffer must be initialized to a fixed length of 100 characters.)</param>
        /// <param name="pName">Name ((String builder buffer must be initialized to a fixed length of 100 characters.))</param>
        /// <param name="pProg">Program number.</param>
        /// <returns></returns>
        [PreserveSig]
        int GetServiceDescriptor(
            [In] int iIndex,
            [Out] StringBuilder pProvider, 
            [Out] StringBuilder pName,    
            [Out] out UInt16 pProg
            );

        /// <summary>
        /// Send either presentation time stamps (PTS) or decoding time stamps (DTS) to decoder.
        /// Normally PTS should be sent to decoder.
        /// </summary>
        /// <param name="set"></param>
        /// <returns></returns>
        [PreserveSig]
        int  SendPTSToH264Dec(
        [In, MarshalAs(UnmanagedType.Bool)] bool set
		);
    }

    // Declare UdpSourceFilter as a COM coclass:
    [ComImport, Guid("43D69FEF-B6A3-4e7e-9949-8EDEDC06CF11")]

    class UdpSourceFilter
    {
    } 
    #endregion

    #region Structures
    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
    public struct parser_TeletextDesc
    {
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 4)]
        public string Language;
        public System.UInt16 Type;
        public System.Byte MagazineNumber;
        public System.Byte PageNumber;
    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
    public struct parser_SubtitleDesc
    {
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 4)]
        public string Language;
        public System.UInt16 Type;
        public System.UInt16 CompositionPageId;
        public System.UInt16 AncillaryPageId;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct parser_VideoInfo
    {
        public System.Int32 Width;
        public System.Int32 Height;
        public System.UInt32 AspectRatioX;
        public System.UInt32 AspectRatioY;
        public System.Int64 AvgTimePerFrameNom;
        public System.Int64 AvgTimePerFrameDenom;
        public System.UInt32 Profile;
        public System.UInt32 Level;
        public System.UInt32 Bitrate;
        public System.UInt16 InterlacedIndicator;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct parser_AudioInfo
    {
        public System.UInt16 ChannelCount;
        public System.UInt16 MpegLayer;
        public System.UInt16 MpegVersion;
        public System.UInt32 SampFrequency;
        public System.UInt32 Bitrate;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct parser_StreamInfo
    {
        public System.UInt16 StreamType;
        public System.UInt16 PID;
        [MarshalAs(UnmanagedType.Bool)]
        public System.Boolean isPESSupported;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 4)]
        public string Language;
        public System.Int32 SubtitlingDescCount;
        public System.Int32 TeletextDescCount;
    }

    public enum StreamType
    {
        VideoMpeg1 = (UInt16)0x01,
        VideoMpeg2 = (UInt16)0x02,
        VIdeoH264 = (UInt16)0x1b,
        AudioMpeg1 = (UInt16)0x03,
        AudioMpeg2 = (UInt16)0x04,
        AudioDolby = (UInt16)0x81,
        PrivateDvbSub = (UInt16)0x8801,
        PrivateTTXSub = (UInt16)0x8802,
        PrivateTTXPage = (UInt16)0x8803
    }

    public enum SubtitleType
    {
        EBUTeletext = (ushort)0x01,
        EBUAsocTeletext = (ushort)0x02,
        DVB_NoAr = (ushort)0x10,
        DVB_4_3 = (ushort)0x11,
        DVB_16_9 = (ushort)0x12,
        DVB_2_21 = (ushort)0x13,
        DVB_HI_NoAr = (ushort)0x20,
        DVB_HI_4_3 = (ushort)0x21,
        DVB_HI_16_9 = (ushort)0x22,
        DVB_HI_2_21 = (ushort)0x23,
    }

    internal enum TeletextType
    {
        InitialTTXPage = (ushort)0x01,
        TeletextSubtitlePage = (ushort)0x02,
        AditionalInformationPage = (ushort)0x03,
        ProgrameSchedulePage = (ushort)0x04,
        TeletextSubtitleHI = (ushort)0x05
    }

    internal enum InterlaceFlag
    { 
        Unknown,
        Progressive,
        TopFirst,
        BottomFirst,
        TopBottomRep,
        BottomTopRep,
        TopBottomTopRep,
        BottomTopBottomRep,
        FrameDoubling,
        FrameTripling,
    }

    #endregion
    #endregion

    #region Parsing helper clases
    public class TSParser
    {
        public const int MAX_SERVICE_NAME_LEN = 100;
        
        // Ds messages
        public const int EC_PARSER_COMPLETE = 0xC3500;
        public const int EC_MSG_Discontinuity = 0xC3501;
        public const int EC_PARSER_STARTED = 0xC3502;
        public const int EC_MSG_FirstAudioPacket = 0xC3510;
        public const int EC_MSG_FirstVideoPacket = 0xC3511;
        public const int EC_MSG_ParserRecvdFirstSample = 0xC3512;
        public const int EC_MSG_Delay =0xC3520;
        public const int EC_MSG_Resync = 0xC3521;
        public const int EC_MSG_AspectRatioChanged = 0xC3530;
        public const int EC_MSG_VideoDimensionChanged = 0xC3531;
        public const int EC_MSG_SERVICE_LIST_RECEIVED = 0xC3550;

        // Error codes
        public const int ERR_RECV_IPAGN_AND_IPV4_BIND_FAILED = 0x801;

        // Success codes
        public const int ALL_PINS_SUCCESS = 0x2;
        public const int VIDEO_PIN_SUCCESS = 0x4;
        public const int AUDIO_PIN_SUCCESS = 0x8;
        public const int SUB_PIN_SUCCESS = 0x10;
        public const int TTX_PIN_SUCCESS = 0x20;

        // Other codes
        public const int PLAYBACK_MODE = 1;
        public const int PARSING_MODE = 2;
        
        public static readonly Dictionary<ushort, string> StreamTypDesc = new Dictionary<ushort, string>()
       {
           {(ushort)StreamType.AudioDolby, "Dolby"},
           {(ushort)StreamType.AudioMpeg1, "MPEG1"},
           {(ushort)StreamType.AudioMpeg2, "MPEG2"},
           {(ushort)StreamType.PrivateDvbSub, "DVB"},
           {(ushort)StreamType.PrivateTTXPage, "Teletext page"},
           {(ushort)StreamType.PrivateTTXSub, "Teletext"},
           {(ushort)StreamType.VIdeoH264, "H264"},
           {(ushort)StreamType.VideoMpeg1, "MPEG1"},
           {(ushort)StreamType.VideoMpeg2, "MPEG2"},
       };

        public static readonly Dictionary<ushort, string> SubtitleTypDesc = new Dictionary<ushort, string>()
       {
           {(ushort)SubtitleType.DVB_16_9, "DVB (16:9)"},
           {(ushort)SubtitleType.DVB_2_21, "DVB (2.12:1)"},
           {(ushort)SubtitleType.DVB_4_3, "DVB (4:3)"},
           {(ushort)SubtitleType.DVB_HI_16_9, "DVB Hearing Impaired (16:9)"},
           {(ushort)SubtitleType.DVB_HI_2_21, "DVB (2.12:1)"},
           {(ushort)SubtitleType.DVB_HI_4_3, "DVB (4:3)"},
           {(ushort)SubtitleType.DVB_HI_NoAr, "DVB Hearing Impaired"},
           {(ushort)SubtitleType.DVB_NoAr, "DVB"},
           {(ushort)SubtitleType.EBUTeletext, "Teletext"},
           {(ushort)SubtitleType.EBUAsocTeletext, "Teletext"},
       };

        const int PRIVATE_PES = 6;
        private IUdpMulticast pFIlterInterface;
        private List<TSInfo> progs;
        public bool DataCollected = false;

        public TSParser(IUdpMulticast fi)
        {
            pFIlterInterface = fi;
            progs = new List<TSInfo>();
        }

        /// <summary>
        /// Get basic info needed to start DS graph
        /// </summary>
        public void GetInfo()
        {
            int programes;
            pFIlterInterface.GetProgCount(out programes);
            for (int i = 0; i < programes; i++)
            {
                TSInfo ti = new TSInfo();
                int streamCount = 0;
                pFIlterInterface.GetPidCount(i, out streamCount);
                for (int j = 0; j < streamCount; j++)
                {
                    ArrangeData(i, j, ref ti);
                }
                progs.Add(ti); // IPTV has usually only one.
            }
            DataCollected = true;
        }

        public List<KeyValuePair<int, int>> GetPATList()
        { 
            int count=0;
            List<KeyValuePair<int, int>> PATList = new List<KeyValuePair<int, int>>();
            pFIlterInterface.GetProgCount(out count);
            for (int i = 0; i < count; i++)
            { 
                ushort progNum, pid;
                pFIlterInterface.GetPATPid(i, out progNum, out pid);
                if (pid != 16) // NIT table
                    PATList.Add(new KeyValuePair<int, int>((int)progNum, (int)pid));
            }
            return PATList;
        }

       
        public List<List<KeyValuePair<int, string>>> GetPMTList()
        {
            List<List<KeyValuePair<int, string>>> PMTLIstAll = new List<List<KeyValuePair<int, string>>>();

            int progcount;
            pFIlterInterface.GetProgCount(out progcount);

            for (int i = 0; i < progcount; i++)
            {
                List<KeyValuePair<int, string>> PMTList = new List<KeyValuePair<int, string>>();
                int streamcount;
                pFIlterInterface.GetPidCount(i, out streamcount);
                for (int j = 0; j < streamcount; j++)
                {
                    ushort pid;
                    int type;
                    pFIlterInterface.GetPMTType(i, j, out pid, out type);
                    PMTList.Add(new KeyValuePair<int, string>((int)pid, StreamTypeToString(type)));
                 }
                PMTLIstAll.Add(PMTList);
            }

            return PMTLIstAll;
        }

        public List<KeyValuePair<int, string>> GetPMTList(int ProgIndex)
        {
            int count=0;
            List<KeyValuePair<int, string>> PMTList = new List<KeyValuePair<int, string>>();
            pFIlterInterface.GetPidCount(ProgIndex, out count);
            for (int i = 0; i < count; i++)
            { 
                ushort pid;
                int type;
                pFIlterInterface.GetPMTType(ProgIndex, i, out pid, out type);
                PMTList.Add(new KeyValuePair<int, string>((int)pid, StreamTypeToString(type)));
            }
            return PMTList;
        }

        public List<ServiceDescriptorTriplet> GetServiceDescriptor()
        {
            List<ServiceDescriptorTriplet>servcdesc = new List<ServiceDescriptorTriplet>();
            
            int count =0;
            pFIlterInterface.ServicesCount(out count);
            
            for (int i = 0; i < count; i++)
            {
                ushort prog;
                StringBuilder provider = new StringBuilder(MAX_SERVICE_NAME_LEN);
                StringBuilder name = new StringBuilder(MAX_SERVICE_NAME_LEN);
                pFIlterInterface.GetServiceDescriptor(i,provider,name,out prog);
                
                Byte[] bufa = Encoding.Unicode.GetBytes(provider.ToString());
                String provider_s = Encoding.ASCII.GetString(bufa);
                Byte[] bufb = Encoding.Unicode.GetBytes(name.ToString());
                String name_s = Encoding.ASCII.GetString(bufb);

                ServiceDescriptorTriplet sd = new ServiceDescriptorTriplet();
                sd.Prog = (int)prog;
                sd.ServiceName = name_s;
                sd.ServiceProvider = provider_s;
                servcdesc.Add(sd);
            }

            return servcdesc;
        }

        private void ArrangeData(int progIndex, int streamIndex, ref TSInfo ti)
        {
            parser_StreamInfo StreamInfo = new parser_StreamInfo();
            pFIlterInterface.GetParsedData(progIndex, streamIndex, ref StreamInfo);

            switch (StreamInfo.StreamType)
            {
                case (ushort)StreamType.VideoMpeg1:
                case (ushort)StreamType.VideoMpeg2:
                case (ushort)StreamType.VIdeoH264:
                    TSVIdeoInfo tsvi = new TSVIdeoInfo();
                    parser_VideoInfo pvi = new parser_VideoInfo();
                    pFIlterInterface.GetVideoInfo(progIndex, streamIndex, ref pvi);

                    tsvi.Type = StreamInfo.StreamType;
                    tsvi.Language = StreamInfo.Language;
                    tsvi.PID = StreamInfo.PID;
                    tsvi.Bitrate = pvi.Bitrate;
                    tsvi.AspectRatioX = pvi.AspectRatioX;
                    tsvi.AspectRatioY = pvi.AspectRatioY;
                    if (pvi.AvgTimePerFrameDenom!=0)
                        tsvi.AvgTimePerField = pvi.AvgTimePerFrameNom/ pvi.AvgTimePerFrameDenom;
                    tsvi.Height = pvi.Height;
                    tsvi.InterlacedFlag = pvi.InterlacedIndicator;
                    tsvi.Level = pvi.Level;
                    tsvi.Profile = pvi.Profile;
                    tsvi.Width = pvi.Width;
                    ti.Video.Add(tsvi);
                    break;

                case (ushort)StreamType.AudioMpeg1:
                case (ushort)StreamType.AudioMpeg2:
                case (ushort)StreamType.AudioDolby:
                    TSAudioInfo tsai = new TSAudioInfo();
                    parser_AudioInfo pai = new parser_AudioInfo();
                    pFIlterInterface.GetAudioInfo(progIndex, streamIndex, ref pai);

                    tsai.Type = StreamInfo.StreamType;
                    tsai.Language = StreamInfo.Language;
                    tsai.PID = StreamInfo.PID;
                    tsai.BoundSubLang = "";
                    tsai.Bitrate = pai.Bitrate;
                    tsai.MpegLayer = pai.MpegLayer;
                    tsai.NumberOfChannels = pai.ChannelCount;
                    tsai.SampFrequency = pai.SampFrequency;

                    if (StreamInfo.SubtitlingDescCount > 0)
                    {
                        parser_SubtitleDesc boundpsd = new parser_SubtitleDesc();
                        pFIlterInterface.GetSubDesc(progIndex, streamIndex, 0, ref boundpsd);
                        tsai.BoundSubLang = boundpsd.Language;
                    }
                    ti.Audio.Add(tsai);
                    break;

                case (ushort)PRIVATE_PES:

                    int SdCount = StreamInfo.SubtitlingDescCount;
                    int TdCount = StreamInfo.TeletextDescCount;

                    // Subtitles
                    for (int s = 0; s < SdCount; s++)
                    {
                        parser_SubtitleDesc psd = new parser_SubtitleDesc();
                        pFIlterInterface.GetSubDesc(progIndex, streamIndex, s, ref psd);

                        if ((psd.Type >= (ushort)SubtitleType.DVB_NoAr && psd.Type <= (ushort)SubtitleType.DVB_2_21)
                            || (psd.Type >= (ushort)SubtitleType.DVB_NoAr && psd.Type <= (ushort)SubtitleType.DVB_2_21)
                            || psd.Type == (ushort)SubtitleType.EBUTeletext
                            )
                        {
                            TSSubtitleInfo tssi = new TSSubtitleInfo();
                            if (psd.Language.Length == 0)
                                tssi.Language = StreamInfo.Language;
                            else
                                tssi.Language = psd.Language;
                            tssi.PID = StreamInfo.PID;
                            tssi.Type = psd.Type;
                            ti.Sub.Add(tssi);
                        }
                    }

                    for (int t = 0; t < TdCount; t++)
                    {
                        parser_TeletextDesc ptd = new parser_TeletextDesc();
                        pFIlterInterface.GetTtxDesc(progIndex, streamIndex, t, ref ptd);

                        // Teletext
                        if (ptd.Type == (ushort)TeletextType.InitialTTXPage)
                        {
                            TSTeletextInfo tsti = new TSTeletextInfo();
                            if (ptd.Language.Length == 0)
                                tsti.Language = StreamInfo.Language;
                            else
                                tsti.Language = ptd.Language;
                            tsti.PID = StreamInfo.PID;
                            tsti.Type = ptd.Type;
                            ti.TTX.Add(tsti);
                        }
                        // Teletext ebu data (subtitles)
                        else if (ptd.Type == (ushort)TeletextType.TeletextSubtitlePage || ptd.Type == (ushort)TeletextType.TeletextSubtitleHI)
                        {
                            TSSubtitleInfo tssi = new TSSubtitleInfo();
                            tssi.Language = ptd.Language;
                            tssi.PID = StreamInfo.PID;
                            tssi.Type = ptd.Type;
                            tssi.TTXMAgNumber = ptd.MagazineNumber;
                            tssi.TTXPageNumber = ptd.PageNumber;
                            ti.Sub.Add(tssi);
                        }
                    }
                    break;
            }
        }

        private string StreamTypeToString(int type)
        {
            string answer = type.ToString();
            switch (type)
            {
                case 0x1 :
                    answer = "ISO/IEC 11172 Video";
                    break;
                case 0x2 :
                    answer = "ISO/IEC 13818-2 Video";
                    break;
                case 0x3:
                    answer = "ISO/IEC 11172 Audio";
                    break;
                case 0x4:
                    answer = "ISO/IEC 13818-3 Audio";
                    break;
                case 0x5:
                    answer = "ISO/IEC 13818-1 private_sections";
                    break;
                case 0x6:
                    answer = "ISO/IEC 13818-1 PES packets containing private data";
                    break;
                case 0x7:
                    answer = "ISO/IEC 13522 MHEG";
                    break;
                case 0x8:
                    answer = "ISO/IEC 13818-1 Annex A DSM-CC";
                    break;
                case 0x9:
                    answer = "ITU-T Rec. H.222.1";
                    break;
                case 0x0a :
                    answer = "ISO/IEC 13818-6 type A";
                    break;
                case 0x0b :
                    answer = "ISO/IEC 13818-6 type B";
                    break;
                case 0x0c :
                    answer = "ISO/IEC 13818-6 type C";
                    break;
                case 0x0d:
                    answer = "ISO/IEC 13818-6 type D";
                    break;
                case 0x0e:
                    answer = "ISO/IEC 13818-1 auxiliary";
                    break;
                case 0x0f :
                    answer = "ISO/IEC 13818-7 Audio with ADTS transport syntax";
                    break;
                case 0x10 :
                    answer = "ISO/IEC 14496-2 Visual";
                    break;
                case 0x11:
                    answer = "ISO/IEC 14496-3 Audio with the LATM transport syntax";
                    break;
                case 0x12:
                    answer = "ISO/IEC 14496-1 SL-packetized stream or FlexMux stream carried in PES packets";
                    break;
                case 0x13:
                    answer = "ISO/IEC 14496-1 SL-packetized stream or FlexMux stream carried in ISO/IEC14496_sections";
                    break;
                case 0x14:
                    answer = "ISO/IEC 13818-6 Synchronized Download Protocol";
                    break;
                default:
                    if (type >= 0x15 && type <= 0x7f)
                        answer = "ISO/IEC 13818-1 Reserved";
                    else if (type >= 0x80 && type <= 0xff)
                        answer = "User Private";
                    break;
            
            }
            return answer;
        }

        public List<TSInfo> ProgrameList
        {
            get { return progs; }
        }

    }

    public class TSInfo
    {
        private List<TSVIdeoInfo> video;
        private List<TSAudioInfo> audio;
        private List<TSSubtitleInfo> sub;
        private List<TSTeletextInfo> ttx;

        public TSInfo()
        {
            video = new List<TSVIdeoInfo>();
            audio = new List<TSAudioInfo>();
            sub = new List<TSSubtitleInfo>();
            ttx = new List<TSTeletextInfo>();
        }

        public List<TSVIdeoInfo> Video
        {
            get { return video; }
            set { video = value; }
        }
        public List<TSAudioInfo> Audio
        {
            get { return audio; }
            set { audio = value; }
        }
        public List<TSSubtitleInfo> Sub
        {
            get { return sub; }
            set { sub = value; }
        }
        public List<TSTeletextInfo> TTX
        {
            get { return ttx; }
            set { ttx = value; }
        }
    }

    public class TSCommonInfo
    {
        private ushort pid;
        private string language;
        private UInt16 type;

        public ushort PID
        {
            get { return pid; }
            set { pid = value; }
        }

        public string Language
        {
            get { return language; }
            set { language = value; }
        }

        public UInt16 Type
        {
            get { return type; }
            set { type = value; }
        }
    }

    public class TSVIdeoInfo : TSCommonInfo
    {
        private int width;
        private int height;
        private uint aspectx;
        private uint aspecty;
        private Int64 avgtimeperframe;
        private uint profile;
        private uint level;
        private uint bitrate;
        private uint interlaced;
        private int seqHeaderSize;
        private byte[] seqHeader;

        public int Width
        {
            get { return width; }
            set { width = value; }
        }
        public int Height
        {
            get { return height; }
            set { height = value; }
        }
        public uint AspectRatioX
        {
            get { return aspectx; }
            set { aspectx = value; }
        }
        public uint AspectRatioY
        {
            get { return aspecty; }
            set { aspecty = value; }
        }
        public Int64 AvgTimePerField
        {
            get { return avgtimeperframe; }
            set { avgtimeperframe = value; }
        }
        public uint Profile
        {
            get { return profile; }
            set { profile = value; }
        }
        public uint Level
        {
            get { return level; }
            set { level = value; }
        }
        public uint Bitrate
        {
            get { return bitrate; }
            set { bitrate = value; }
        }
        public uint InterlacedFlag
        {
            get { return interlaced; }
            set { interlaced = value; }
        }
        public int SequenceHeaderSize
        {
            get { return seqHeaderSize; }
            set { seqHeaderSize = value; }
        }
        public byte[] SequenceHeader
        {
            get { return seqHeader; }
            set { seqHeader = value; }
        }
    }

    public class TSAudioInfo : TSCommonInfo
    {
        private int channelnumber;
        private uint sampfreq;
        private uint bitrate;
        private int mpeglayer;
        private string boundSubLang;

        public int NumberOfChannels
        {
            get { return channelnumber; }
            set { channelnumber = value; }
        }
        public uint SampFrequency
        {
            get { return sampfreq; }
            set { sampfreq = value; }
        }
        public uint Bitrate
        {
            get { return bitrate; }
            set { bitrate = value; }
        }
        public int MpegLayer
        {
            get { return mpeglayer; }
            set { mpeglayer = value; }
        }
        public string BoundSubLang
        {
            get { return boundSubLang; }
            set { boundSubLang = value; }
        }
    }

    public class TSSubtitleInfo : TSCommonInfo
    {
        private int ttxmagNumber = 0;
        private int ttxpageNumber = 0;
        private ushort dvbCompositionPageId = 0;
        private ushort dvbAncillaryPageId=0;

        public int TTXMAgNumber
        {
            get { return ttxmagNumber; }
            set { ttxpageNumber = value; }
        }
        public int TTXPageNumber
        {
            get { return ttxpageNumber; }
            set { ttxpageNumber = value; }
        }
        public ushort DVBCompositionPageId
        {
            get { return dvbCompositionPageId; }
            set {dvbCompositionPageId=value;}
        }
        public ushort DVBAncillaryPageId
        {
            get { return dvbAncillaryPageId; }
            set { dvbAncillaryPageId = value; }
        }
    }

    public class TSTeletextInfo : TSCommonInfo
    {
        // Only type and language.
    }

    public class ServiceDescriptorTriplet
    {
        private int prog;
        private string serviceProvider;
        private string serviceName;

        public int Prog
        {
            get { return prog; }
            set{ prog = value;}
        }
        public string ServiceProvider
        {
            get { return serviceProvider; }
            set { serviceProvider = value; }        
        }
        public string ServiceName
        {
            get { return serviceName; }
            set { serviceName = value; }
        }
    }

    #endregion

    #region General helper class
    public static class DSHelper
    {
        public static int ToFourCC(string FourCC)
        {
            if (FourCC.Length != 4)
            {
                throw new Exception("FourCC strings must be 4 characters long " + FourCC);
            }
            int result = ((int)FourCC[3]) << 24
                        | ((int)FourCC[2]) << 16
                        | ((int)FourCC[1]) << 8
                        | ((int)FourCC[0]);

            return result;
        }

        /// <summary>
        /// Convert Mpeg-2 video profile to directshow profile.
        /// </summary>
        /// <param name="Profile"></param>
        /// <returns></returns>
        public static int ToDsMpeg2Profile(int Profile)
        {
            return 6 - Profile;
        }

        /// <summary>
        /// Convert Mpeg-2 video level to directshow level.
        /// </summary>
        /// <param name="Level"></param>
        /// <returns></returns>
        public static int ToDsMpeg2Level(int Level)
        {
            int level = Level;
            switch (level)
            {
                case 6:
                    level = 3;
                    break;

                case 8:
                    level = 2;
                    break;
                case 10:
                    level = 1;
                    break;
            }
            return level;
        }

        public static string Mpeg2ProfileDesc(int n)
        {
            string answer = n.ToString();
            switch (n)
            {
                case 1:
                    return "High";
                case 2:
                    return "Spatial";
                case 3: 
                    return "SNR";
                case 4:
                    return "Main";
                case 5:
                    return "Simple";
            }
            return answer;
        }

        public static string H264ProfileDesc(int n)
        {
            
            string answer = n.ToString();
            switch (n)
            {
                case 77:
                    return "Main";
                case 66:
                    return "Baseline";
                case 100:
                    return "High";
                case 88:
                    return "Extended";
            }
            return answer;
        }
    } 

    #endregion
}