﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.IO;
using System.Xml;

namespace MulticastTV
{
    static class ExportToXspf
    {
        static public void Start(string file, string IpPrefix, int extension)
        {
            if (IpPrefix == string.Empty)
                IpPrefix = "udp://@";
            
            DataView dw = ChannelTable.menu.Tables["Menu"].DefaultView;
            dw.Sort ="Chan";

            EnsureUniqueChanNumbers(dw);

            List<Node> list = new List<Node>();
   
            XmlTextWriter textWriter = new XmlTextWriter(file, Encoding.UTF8);

            textWriter.Formatting =  Formatting.Indented;

            textWriter.WriteStartDocument();
            //textWriter.WriteComment("Playlist created with MulticastTV playlist editor");
            textWriter.WriteStartElement("playlist","http://xspf.org/ns/0/");
            textWriter.WriteAttributeString("version", "1");
            textWriter.WriteAttributeString("xmlns:vlc", "http://www.videolan.org/vlc/playlist/0");
            textWriter.WriteAttributeString("xmlns:MulticastTV", "http://sites.google.com/site/multicasttv/Home/playlist/0");
            
            textWriter.WriteStartElement("date");
            textWriter.WriteString(XmlConvert.ToString(DateTime.Now,XmlDateTimeSerializationMode.Local));
            textWriter.WriteEndElement();

            textWriter.WriteStartElement("creator");
            textWriter.WriteString("MulticastTV Playlist Editor");
            textWriter.WriteEndElement();

            textWriter.WriteStartElement("info");
            textWriter.WriteString("http://sites.google.com/site/multicasttv/Home");
            textWriter.WriteEndElement();

            textWriter.WriteStartElement("trackList");
            
            int i = 1; 
            foreach (DataRowView dr in dw)
            {
                //if (dr.IsNew)
                //    continue;
                
                string ip = dr[0].ToString();
                int ch = (int)dr[1];
                string name = dr[2].ToString();
                string audio = dr[4].ToString().Split(",".ToCharArray())[0]; ;
                string group = dr[3].ToString();
                string epgid = dr[5].ToString();

                if (group != "")
                {
                    string[] groups = group.Split(",".ToCharArray());
                    foreach (string grp in groups)
                    {
                        int nodeindex = list.FindIndex(delegate(Node n) { return n.Group == grp; });
                        if (nodeindex == -1)
                        {
                            Node nnode = new Node(grp);
                            nnode.Channels.Add(ch.ToString());
                            list.Add(nnode);
                        }
                        else
                        {
                            list[nodeindex].Channels.Add(ch.ToString());
                        }
                    }
                }

                textWriter.WriteStartElement("track");

                textWriter.WriteStartElement("location");
                textWriter.WriteString(IpPrefix+ip);
                textWriter.WriteEndElement(); // End location

                textWriter.WriteStartElement("title");
                textWriter.WriteString(name);
                textWriter.WriteEndElement(); // End title
                
                textWriter.WriteStartElement("trackNum");
                textWriter.WriteString(i.ToString());
                textWriter.WriteEndElement(); // End tracknum

                if (extension == 0 || extension == 2)
                {
                    // MulticastTV extension
                    textWriter.WriteStartElement("extension");
                    textWriter.WriteAttributeString("application", "http://sites.google.com/site/multicasttv/Home/playlist/0");

                    textWriter.WriteStartElement("MulticastTV:chan");
                    textWriter.WriteString(ch.ToString());
                    textWriter.WriteEndElement(); // End chan
                    if (group != "")
                    {
                        textWriter.WriteStartElement("MulticastTV:group");
                        textWriter.WriteString(group);
                        textWriter.WriteEndElement(); // End group
                    }
                    if (audio != "")
                    {
                        textWriter.WriteStartElement("MulticastTV:lang");
                        textWriter.WriteString(audio);
                        textWriter.WriteEndElement(); // End lang
                    }
                    if (epgid != "")
                    {
                        textWriter.WriteStartElement("MulticastTV:epgid");
                        textWriter.WriteString(epgid);
                        textWriter.WriteEndElement(); // End epgid
                    }
                    textWriter.WriteEndElement(); // End  MulticastTV extension

                }

                if (extension == 0 || extension == 1)
                {
                    // VLC extension
                    textWriter.WriteStartElement("extension");
                    textWriter.WriteAttributeString("application", "http://www.videolan.org/vlc/playlist/0");

                    textWriter.WriteStartElement("vlc:id");
                    textWriter.WriteString(ch.ToString());
                    textWriter.WriteEndElement(); // End id
                    textWriter.WriteEndElement(); // End VLC extension
                }

                textWriter.WriteEndElement(); // End track

                i += 1;
            }

            textWriter.WriteEndElement(); // end tracklist

            if (extension == 0 || extension == 1)
                createVlcNodesExtension(textWriter, list);

            textWriter.WriteEndElement(); // end playlist
            
            textWriter.WriteEndDocument();

            textWriter.Close();            
        }

        static public void EnsureUniqueChanNumbers(DataView dw)
        {
            int prevch=0;
            foreach (DataRowView dr in dw)
            {
                if ((int)dr["Chan"] <= prevch)
                {
                    dr["Chan"] = prevch + 1;
                } 
                prevch = (int)dr["Chan"];
            }
        }

        static void createVlcNodesExtension(XmlTextWriter textWritter, List<Node> list)
        {
            if (list.Count > 0)
            {
                textWritter.WriteStartElement("extension");
                textWritter.WriteAttributeString("application", "http://www.videolan.org/vlc/playlist/0");

                foreach (Node nd in list)
                {
                    textWritter.WriteStartElement("vlc:node");
                    textWritter.WriteAttributeString("title", nd.Group);

                    foreach (string ch in nd.Channels)
                    {
                        textWritter.WriteStartElement("vlc:item");
                        textWritter.WriteAttributeString("tid", ch);
                        textWritter.WriteEndElement(); // vlc:item end
                    }

                    textWritter.WriteEndElement(); //vlc:_node end
                }

                textWritter.WriteEndElement(); // end extension
            }
        }

    }

    public class Node
    {
        string group;
        List<string> channels;

        public Node(string grp)
        {
            group = grp;
            channels = new List<string>();
        }

        public string Group
        {
            get { return group; }
        }

        public List<string> Channels
        {
            get { return channels; }
        }

        public void AddChannels(string ch)
        {
            channels.Add(ch);
        }
    }


}
