﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Xml;
using System.IO;

namespace MulticastTV
{

    /// <summary>
    /// Import from AditeyaTV(atvlist), m3u, csv or excel(xls) file format
    /// Here you can add import functions for other formats !!!
    /// Output datatable has to be in AditeyatTV format
    /// </summary>
    public static class Import 
    {
        static char[] delimiterChars1 = { ';' };
        static char[] delimiterChars2 = { ',' };
        static char[] delimiterChars3 = { ':' };
        static char[] delimiterChars4 = { '-' };
        static char[] delimiterChars5 = { '@' };

        /// <summary>
        /// Returns TEMPORARY DataTable!!! One channel name can have MULTIPLE ip numbers - look at aditeyaTV format.
        /// </summary>
        /// <param name="path">Startup path</param>
        /// <param name="clean">Common aplication folder</param>
        /// <param name="clean">File to import from</param>
        public static DataTable array(string path, string appath, string file, ref bool NeedsAdditionalInfo)
        {

            DataTable dt = new DataTable();

            dt.Columns.Add("Channel", typeof(int));
            dt.Columns.Add("Name", typeof(string));
            dt.Columns.Add("Group", typeof(string));  // Multiple groups must be separated by a comma.
            dt.Columns.Add("Locked", typeof(string)); // "0" or "1"
            dt.Columns.Add("Ip", typeof(string[]));   // language+ip:port example: [english+239.255.0.108:5002,czech+239.255.0.179:5002]

            // Optional EpgID column
            DataColumn column;
            column = new DataColumn();
            column.DataType = typeof(string);
            column.ColumnName = "EpgId";
            column.ReadOnly = false;
            column.Unique = false;
            column.DefaultValue = "";
            dt.Columns.Add(column);

            string ext = Path.GetExtension(file).ToUpper();

            if (ext == ".XLS")
            {
                NeedsAdditionalInfo = false;
                ImportFromExcelAndCsv exc = new ImportFromExcelAndCsv(file);
                dt = exc.FromExcell(dt,true);
            }

            else if (ext == ".CSV")
            {
                NeedsAdditionalInfo = false;
                ImportFromExcelAndCsv exc = new ImportFromExcelAndCsv(file);
                dt = exc.fromCsv(dt);
            }

            else if (ext == ".ATVLIST")
            {
                NeedsAdditionalInfo = false;
                FillDtFromAditeyaTVFormat(file, dt);
            }

            else if (ext == ".M3U")
            {
                NeedsAdditionalInfo = true;
                FillDtFromM3u(file, dt, ref NeedsAdditionalInfo);
            }
            else if (ext == ".XSPF")
            {
                NeedsAdditionalInfo = true;
                FillDtFromXspf(file, dt, ref NeedsAdditionalInfo);
            }

            else if (ext == ".JS")
            {
                NeedsAdditionalInfo = false;
                FillDtFromSimplicityJs(file, dt);
            }

            return dt;
        }

        //=======================================================================================
        // AditeyaTV
        //=======================================================================================
        static void FillDtFromAditeyaTVFormat(string file, DataTable dt)
        {
            string line;
            StreamReader SR = File.OpenText(file);
            line = SR.ReadLine();
            while (line != null)
            {
                if (line.Length > 28)
                {
                    int channel = int.Parse(line.Split(delimiterChars1)[0].Substring(8));
                    string name = line.Split(delimiterChars1)[1].Substring(5);
                    string locked = line.Split(delimiterChars1)[2].Substring(7);
                    string group = line.Split(delimiterChars1)[3].Substring(6);
                    string audiotemp = line.Split(delimiterChars1)[4].Substring(6);
                    string[] audio = audiotemp.Split(delimiterChars2);
                    dt.Rows.Add(channel, name, group, locked, audio);
                }
                line = SR.ReadLine();
           }
           SR.Close();
        }
        //=======================================================================================


        //=======================================================================================
        // Xspf 
        //=======================================================================================
        static void FillDtFromXspf(string file, DataTable dt, ref bool NeedsAdditionalInfo)
        {
            XmlTextReader textReader = null;
            try
            {
                textReader = new XmlTextReader(file);
            }
            catch { return; }

            while (textReader.Read())
            {
                switch (textReader.NodeType)
                {
                    case XmlNodeType.Element:
                        if (textReader.Name == "trackList")
                            parseXspfTrackList(textReader, dt, ref NeedsAdditionalInfo);
                        else if (textReader.Name == "extension")
                            parseExtensions(textReader, new ExtensionData(), dt, ref NeedsAdditionalInfo);
                     break;
                }
            }
        }

        static void parseXspfTrackList(XmlTextReader textReader , DataTable dt, ref bool NeedsAdditionalInfo)
        { 
            int counter = 0;
            while (textReader.Read())
            {
                switch (textReader.NodeType)
                {
                    case XmlNodeType.Element: 
                        if (textReader.Name == "track")
                            parseXspfTrack(textReader, dt,ref counter, ref NeedsAdditionalInfo);
                        break;

                    case XmlNodeType.EndElement:
                        if (textReader.Name == "trackList")
                            return;
                        break;
                }
            }
        }

        static void parseXspfTrack(XmlTextReader textReader , DataTable dt, ref int counter, ref bool Additionalinfo)
        {

            string ip = "";
            string name = "";
            ExtensionData ed = new ExtensionData();;
            counter += 1;
            ed.num = counter;
            bool ChanNumFound = false;

            while (textReader.Read())
            {
                switch (textReader.NodeType)
                {
                    case XmlNodeType.Element:

                        if (textReader.Name == "location")
                        {
                            textReader.Read();
                            ip = textReader.Value;
                        }
                        else if (textReader.Name == "title")
                        {
                            textReader.Read();
                            name = textReader.Value;
                        }
                        else if (textReader.Name == "trackNum" && !ChanNumFound)
                        {
                            textReader.Read();
                            int.TryParse(textReader.Value, out ed.num);
                        }

                        else if (textReader.Name == "extension")
                        {
                            parseExtensions(textReader, ed,dt,ref Additionalinfo);
                            if (ed.hasChanNumInfo)
                                ChanNumFound = true;
                        }
                        break;

                    case XmlNodeType.EndElement:
                        if (textReader.Name == "track")
                        { 
                            int ip_start = findAddressStart(ip);
                            if (ip_start != -1)
                            {
                                ip = ip.Substring(ip_start);
                                ip = ed.lang+ "+" + ip;
                                dt.Rows.Add(ed.num, name, ed.group, false, new string[] { ip },ed.epgid);
                            }
                            return;
                        }
                        break;
                }
            }
         }

        static void parseExtensions(XmlTextReader textReader, ExtensionData ed, DataTable dt, ref bool NeedsAdditionalInfo)
        {
            while (textReader.MoveToNextAttribute()) // Read the attributes.
            {
                if (textReader.Name == "application")
                {
                    if (textReader.Value == "http://www.videolan.org/vlc/playlist/0")
                        parseVLCExtension(textReader, ed, dt, ref NeedsAdditionalInfo);
                    else if (textReader.Value == "http://sites.google.com/site/multicasttv/Home/playlist/0")
                    {
                        parseMulticastTVExtension(textReader, ed);
                        NeedsAdditionalInfo = false;
                    }
                }
            }
        }

        static void parseVLCExtension(XmlTextReader textReader, ExtensionData ed, DataTable dt, ref bool NeedsAdditionalInfo)
        {
            while (textReader.Read())
            {
                switch (textReader.NodeType)
                {
                    case XmlNodeType.Element:
                        if (textReader.Name == "vlc:id")
                        {
                            textReader.Read();
                            if (int.TryParse(textReader.Value, out ed.num))
                                ed.hasChanNumInfo = true;
                        }
   
                        else if (textReader.Name == "vlc:node")
                        {
                            parseVLCNodes(textReader,dt);
                            NeedsAdditionalInfo = false;

                        }
                        break;

                    case XmlNodeType.EndElement:
                        if (textReader.Name == "extension")
                            return;
                        break;

                }
            }
        }

        static void parseVLCNodes(XmlTextReader textReader, DataTable dt)
        {
            while (textReader.MoveToNextAttribute())
            {
                if (textReader.Name == "title")
                {
                    // join all possible subnodes with root node
                    string group = textReader.Value;

                    while (textReader.Read())
                    {
                        switch (textReader.NodeType)
                        {
                            case XmlNodeType.Element:
                                if (textReader.Name == "vlc:node") //subnode
                                    parseVLCNodes(textReader,dt);
                                else if (textReader.Name == "vlc:item")
                                    updateGroupsFromVlcNodes(textReader,group,dt);
                                break;

                            case XmlNodeType.EndElement:
                                if (textReader.Name == "vlc:node")
                                    return;
                                break;

                        }
                    }
                }
            }
        }

        static void updateGroupsFromVlcNodes(XmlTextReader textReader,string group, DataTable dt)
        {
            while (textReader.MoveToNextAttribute())
            {
                if (textReader.Name == "tid")
                {
                    string chanId = textReader.Value;
                    // This is slow, but sure.
                    foreach (DataRow dr in dt.Rows)
                    {
                        if (((int)dr["Channel"]).ToString() == chanId)
                        {
                            if ( (string)(dr["Group"])=="" )
                                dr["Group"] = ((string)dr["Group"] + ","+group).TrimStart(",".ToCharArray());
                            break;
                        }
                    }
                }
            }
        }

        static void parseMulticastTVExtension(XmlTextReader textReader, ExtensionData ed)
        {
            while (textReader.Read())
            {
                switch (textReader.NodeType)
                {
                    case XmlNodeType.Element:
                        if (textReader.Name == "MulticastTV:group")
                        {
                            textReader.Read();
                            ed.group=textReader.Value;
                        }
  
                        else if (textReader.Name == "MulticastTV:lang")
                        {
                            textReader.Read();
                            ed.lang=textReader.Value;
                        }

                        else if (textReader.Name == "MulticastTV:chan")
                        {
                            textReader.Read();
                            if (int.TryParse(textReader.Value, out ed.num))
                                ed.hasChanNumInfo = true;
                        }

                        else if (textReader.Name == "MulticastTV:epgid")
                        {
                            textReader.Read();
                            ed.epgid = textReader.Value;
                        }
                        break;

                    case XmlNodeType.EndElement:
                        if (textReader.Name == "extension")
                            return;
                        break;
                }
            }
        }

        //=======================================================================================


        //=======================================================================================
        // M3U - different types
        //=======================================================================================
        static void FillDtFromM3u(string file, DataTable dt, ref bool NeedsAdditionalInfo)
        {
            bool chanfromartist = false;
            string line;
            int i = 1;
            int lastchan = 0;
            int mchannel = 0;
            string mname = "";
            string mlocked = "0";
            string mgroups = "";
            string language = "";
            string epgid = "";
            int counter = 0;
            
            StreamReader SR = File.OpenText(file);
            line = SR.ReadLine();

            if (line.StartsWith("#EXTM3U"))
                line = SR.ReadLine();
            else
                return; //Not a m3u file

            while (line != null)
            {
                if (line.Length == 0)
                {
                    line = SR.ReadLine();
                    continue;
                }

                string options = "";
                if (line.StartsWith("#") && line.Contains(":"))
                    options = line.Split(":".ToCharArray())[0];

                switch (options)
                {
                    case "#EXTINF":

                        // m3u name and number
                        mchannel = lastchan + 1;
                        int parsed = 0;
                        mname = line.Substring(line.IndexOf(",") + 1);
                        
                        // check for format of first entry
                        if (counter == 0 && mname.Contains("-") && int.TryParse((mname.Split(delimiterChars4, 2)[0].Trim()), out parsed))
                            chanfromartist = true;
                         
                        if (chanfromartist)
                        {
                           // Try to get channel number from artist name. 
                            if (int.TryParse((mname.Split(delimiterChars4, 2)[0].Trim()), out parsed))
                            {
                                mname = mname.Split(delimiterChars4, 2)[1];
                                mchannel = parsed;
                            }
                            else
                                chanfromartist = false;
                        }
                        else
                        {
                            // Try to get channel number from length field.
                            int start = line.IndexOf(":") + 1;
                            int stop = line.IndexOf(",");
                            int itchan = 0;
                            string tchan = line.Substring(start, stop - start);
                            if (int.TryParse(tchan, out itchan))
                            {
                                if (itchan != 0)
                                    mchannel = itchan;
                            }
                        }
                        mname = mname.Trim();

                        counter += 1;
                        break;

                    case "#EXTTV":

                        if (NeedsAdditionalInfo == true)
                            NeedsAdditionalInfo = false; // Tano format has all necessary data

                        // Extended Tano format
                        string[] data = line.Split(":".ToCharArray())[1].Split(";".ToCharArray());
                        mgroups = data[0];
                        language = data[1];
                        epgid = data[2];
                        break;

                    case "":

                        int ip_start = findAddressStart(line);
                        if (ip_start !=-1)
                        {
                            line = line.Substring(ip_start);
                            line = "+" + line;
                            line = language + line;
                            dt.Rows.Add(mchannel, mname, mgroups, mlocked, new string[] { line }, epgid);
                            lastchan = mchannel;
                            i++;
                            
                            //reset
                            mname = ""; mlocked = "0"; mgroups = ""; epgid = "";
                        }
                        break;

                }
                line = SR.ReadLine();
            }
            SR.Close();
        }
        // =======================================================================================


        // =======================================================================================
        // Simplicity - no php
        //========================================================================================

        static void FillDtFromSimplicityJs(string file, DataTable dt)
        {
            string line;
            StreamReader SR = File.OpenText(file);
            do
            {  
                line = SR.ReadLine();
                if (line == null)
                {
                    SR.Close();
                    return;
                }

            } while ((!line.StartsWith("la") && !line.StartsWith("MctvRecSrvPlaylist")));
            
            int arrstart = -1;
            arrstart = line.IndexOf("=[");

            if (arrstart != -1)
            {
                line = line.Substring(arrstart+1);
                string[] entries = line.Split("]".ToCharArray());

                int len = entries.Length;

                char[] trimchar = { ',', '[', ']' };
                char[] ph = { '"' };
                for (int i = 0; i < Math.Max(0, len - 2); i++)
                {
                    string strippedentrie = entries[i].Trim(trimchar);
                    string[] entriedata = strippedentrie.Split(",".ToCharArray());

                    int channel = 0;
                    int.TryParse(entriedata[2].Trim(ph), out channel);
                    string name = entriedata[1].Trim(ph);
                    string ip = entriedata[3].Trim(ph);
                    string port = entriedata[4].Trim(ph);
                    string ipport = ip + ":" + port;
                    string group = entriedata[5].Trim(ph);
                    string lang = entriedata[6].Trim(ph);
                    string epgid = entriedata[7].Trim(ph);

                    string audio = lang + "+" + ipport;

                    dt.Rows.Add(channel, name, group, false, new string[] { audio }, epgid);
                }
            }
            SR.Close();
        }

        // =======================================================================================



        // =======================================================================================
        // Add your import function HERE:
        //========================================================================================


        // =======================================================================================


        static int findAddressStart(string line)
        {
            // Find multicast type of an address.
            int ip_start = line.LastIndexOf("//@");
            if (ip_start != -1)
            {
                ip_start += 3;
            }
            else
            {
                ip_start = line.LastIndexOf("//");
                if (ip_start != -1)
                {
                    ip_start += 2;
                    // Check for NIC address prefix, separated with "@".
                    int nicseparator = line.IndexOf("@", Math.Max(0, ip_start));
                    if (nicseparator != -1)
                    {
                        ip_start = nicseparator + 1;
                    }
                    else
                    {
                        // Check for some other type of prefixes.
                        int pf_http = line.IndexOf("/http/", ip_start);
                        int pf_udp = line.IndexOf("/udp/", ip_start);
                        if (pf_http != -1)
                            ip_start = pf_http + 6;
                        else if (pf_udp != -1)
                            ip_start = pf_udp + 5;
                    }
                }
            }
            return ip_start;
        }
    }
    
    class ExtensionData
    {
        public string group = "";
        public string lang = "";
        public int num=0 ;
        public string epgid = "";
        public bool hasChanNumInfo = false;
    }

}
